package com.xydl.service.impl;


import com.xydl.mapper.*;
import com.xydl.model.*;
import com.xydl.service.EiaService;
import com.xydl.util.CommonUtils;
import com.xydl.util.DataSourceUtils;
import com.xydl.util.FormatUtil;
import com.xydl.util.MqttUtil;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.sql.*;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.Date;


@Service
public class EiaServiceImpl implements EiaService {
    private  static final Logger logger = LoggerFactory.getLogger(EiaServiceImpl.class);

    private  static final String SYNC_TABLE = "sync_tables_info";

    @Autowired
    private EaifMapper eaifMapper;

    @Autowired
    private EpaMapper epaMapper;

    @Autowired
    private EiaMapper eiaMapper;

    @Autowired
    private EtpMapper etpMapper;

    @Autowired
    private MicMapper micMapper;

    @Autowired
    private MoaMapper moaMapper;

    @Autowired
    private PdMapper pdMapper;

    @Autowired
    private RptTemperMapper rptTemperMapper;

    @Autowired
    private ScurMapper scurMapper;

    @Autowired
    private Sf6Mapper sf6Mapper;

    @Autowired
    private Sf6envMapper sf6envMapper;


    @Override
    public  List<Eaif> getEaif() {
        return eaifMapper.getEaif();
//
    }

    @Override
    public List<Eia> getEia() {
        return eiaMapper.getEia();
    }

    @Override
    public void getEpa() {
        List<String> tableNames = epaMapper.getTableNamesBySyncTable(SYNC_TABLE);

//        for(String tableName : tableNames){
//            System.out.println(tableName);
//            List<Integer> eqmids = epaMapper.getEqmidsByTableName(tableName);
//            for(int eqmid : eqmids){
//                List<Epa> epas = epaMapper.getEpa(eqmid);
//                String jsonStringEpa = FormatUtil.list2Json(epas);
//                MqttUtil.publish2MQTT(jsonStringEpa);
//            }
//        }
        for(String tableName : tableNames){
            String sql = epaMapper.getSqlBySyncTable(SYNC_TABLE,tableName);
            List<Integer> eqmids = epaMapper.getEqmidsByTableName(tableName);
            if(sql.contains("?")){
                for(int eqmid : eqmids){
                    String newSql = sql.replaceAll("\\?",eqmid+"");
                    List<SuperModel> epas = epaMapper.getData(newSql);
                    String jsonStringEpa = FormatUtil.list2Json(epas);
                    MqttUtil.publish2MQTT(jsonStringEpa);
                }
            }
        }
    }

    @Override
    public List<Etp> getEtp() {
        return etpMapper.getEtp();
    }

    @Override
    public List<Microclimate> getMic() {
         return micMapper.getMicroclimate();
    }

    @Override
    public List<Moa> getMoa() {
        return moaMapper.getMoa();
    }

    @Override
    public List<Pd> getPd() {
        return pdMapper.getPd();
    }

    @Override
    public List<RptTemper> getRptTemper() {
        return rptTemperMapper.getRptTemper();
    }

    @Override
    public List<Scur> getScur() {
        return scurMapper.getScur();
    }

    @Override
    public List<Sf6> getSf6() {
        return sf6Mapper.getSf6();
    }

    @Override
    public List<Sf6env> getSf6env() {
        return sf6envMapper.getSf6env();
    }

    public List<String> getAllTableName() {
        Connection conn = null;
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        List<String> tableNames = new ArrayList<>();
        try {
            conn = DataSourceUtils.getConnection();
            String sql = "select table_name from sync_tables_info";
            pstmt = conn.prepareStatement(sql);
            rs = pstmt.executeQuery();

            while(rs.next()){
                tableNames.add(rs.getString("table_name"));
            }
        } catch (SQLException e) {
            logger.error("execute sql exception:", e);
        } finally {
            DataSourceUtils.closeResource(rs, pstmt, conn);
        }

        return tableNames;
    }

    public Map<String,String> getFieldMap(String tableName) {
        Connection conn = null;
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        Map<String,String> fieldsMap = new HashMap<>();
        try {
            conn = DataSourceUtils.getConnection();
            String sql = "select sync_fields_info.field_name, sync_fields_info.dest_field_name " +
                    "from sync_fields_info,sync_tables_info " +
                    "where sync_fields_info.client_id = 10 and sync_fields_info.table_name = sync_tables_info.table_name and sync_tables_info.table_name=?";
            pstmt = conn.prepareStatement(sql);
            pstmt.setString(1, tableName);
            rs = pstmt.executeQuery();
            ResultSetMetaData metaData = rs.getMetaData();

            while(rs.next()){
                fieldsMap.put(rs.getString("field_name"),rs.getString("dest_field_name"));
            }
        } catch (SQLException e) {
            logger.error("execute sql exception:", e);
        } finally {
            DataSourceUtils.closeResource(rs, pstmt, conn);
        }

            return fieldsMap;
    }

//    public String getLastRecordTimeSended(String tableName,String deviceId) {
//        Connection conn = null;
//        PreparedStatement pstmt = null;
//        ResultSet rs = null;
//        Timestamp timeStamp = null;
//        try {
//            conn = DataSourceUtils.getConnection();
//            String sql = "select  field_val2 from sync_records where table_name =? and devid_val=?";
//            pstmt = conn.prepareStatement(sql);
//            pstmt.setString(1, tableName);
//            pstmt.setString(2, deviceId);
//            rs = pstmt.executeQuery();
//            if(rs.next()){
//                timeStamp = rs.getTimestamp("field_val2");
//
//            }
//        } catch (SQLException e) {
//            logger.error("execute sql exception:", e);
//        } finally {
//            DataSourceUtils.closeResource(rs, pstmt, conn);
//        }
//
//        return  new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(timeStamp);
//    }

//    public List<String> getDeviceID(String tableName) {
//        Connection conn = null;
//        PreparedStatement pstmt = null;
//        ResultSet rs = null;
//        String sqlExecuting = null ;
//        List<String> deviceIDs = new ArrayList<>();
//        try {
//            conn = DataSourceUtils.getConnection();
//            String sql = "select distinct devid_val from sync_records where table_name =?";
//            pstmt = conn.prepareStatement(sql);
//            pstmt.setString(1, tableName);
//            rs = pstmt.executeQuery();
//            while(rs.next()){
//                deviceIDs.add(rs.getString("devid_val"));
//            }
//        } catch (SQLException e) {
//            logger.error("execute sql exception:", e);
//        } finally {
//            DataSourceUtils.closeResource(rs, pstmt, conn);
//        }
//
//        return  deviceIDs;
//    }

    public Map<String,Object> getDeviceID1(String tableName) {
        Connection conn = null;
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        String sqlExecuting = null ;
        Map<String,Object> devIDTimeMap = new HashMap<>();
        try {
            conn = DataSourceUtils.getConnection();
            String sql = "select devid_val,field_val2 from sync_records where table_name =?";
            pstmt = conn.prepareStatement(sql);
            pstmt.setString(1, tableName);
            rs = pstmt.executeQuery();
            while(rs.next()){
                devIDTimeMap.put(rs.getString("devid_val"),rs.getString("field_val2"));
            }
        } catch (SQLException e) {
            logger.error("execute sql exception:", e);
        } finally {
            DataSourceUtils.closeResource(rs, pstmt, conn);
        }

        return  devIDTimeMap;
    }

    public String getSQL(String tableName) {
        Connection conn = null;
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        String sqlExecuting = null ;
        try {
            conn = DataSourceUtils.getConnection();
            String sql = "select * from sync_tables_info where table_name =?";
            pstmt = conn.prepareStatement(sql);
            pstmt.setString(1, tableName);
            rs = pstmt.executeQuery();
            if(rs.next()){
                sqlExecuting = rs.getString("sql");
            }
        } catch (SQLException e) {
            logger.error("execute sql exception:", e);
        } finally {
            DataSourceUtils.closeResource(rs, pstmt, conn);
        }

        return  sqlExecuting;
    }

    public List<Map<String,Object>> getData(String sqlExecuting, String deviceId, String timeStamp) {
        Connection conn = null;
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        List<Map<String,Object>> records = new ArrayList<>();
        try {
            conn = DataSourceUtils.getConnection();
            String sql = sqlExecuting;
            pstmt = conn.prepareStatement(sql);
            pstmt.setString(1, deviceId);
            pstmt.setString(2,timeStamp);
            rs = pstmt.executeQuery();
            int columnCount = rs.getMetaData().getColumnCount(); //获取列的数量
            while(rs.next()){
                Map<String,Object> record = new HashMap<>();
                for (int col = 0; col < columnCount; col++) {
                    String columnName = rs.getMetaData().getColumnName(col + 1);
                    String columnValue = rs.getString(columnName);
                    record.put(columnName,columnValue);
                }
                records.add(record);
            }
        } catch (SQLException e) {
            logger.error("execute sql exception:", e);
        } finally {
            DataSourceUtils.closeResource(rs, pstmt, conn);
        }

        return  records;
    }


//    @Scheduled(initialDelay=1000, fixedRate = 1000 * 3600) //通过@Scheduled声明该方法是计划任务，使用fixedRate属性每隔固定时间执行
//    public void reportRecord(){
//        List<String> allTableNames = getAllTableName();
//        Map<String,Map<String,String>> tableFieldMap = new HashMap<>();
//        Map<String, List<Map<String,List<Map<String,Object>>>>> allTableData = new HashMap<>();
//        for(String tablenName : allTableNames){
//            //if用来测试
//            if(!"data_eaif_h".equals(tablenName)){
//                Map<String,String> fieldMap = getFieldMap(tablenName);
//                tableFieldMap.put(tablenName,fieldMap);
//
//                String sqlExecuting = getSQL(tablenName);
//                List<String> deviceIDs = getDeviceID(tablenName);
//                List<Map<String,List<Map<String,Object>>>> dataOfoneTable = new ArrayList<>();
//                for(String deviceID : deviceIDs){
//                    Map<String,List<Map<String,Object>>> deviceIDMap = new HashMap<>();
//                    String timeStamp = getLastRecordTimeSended(tablenName,deviceID);
//                    List<Map<String,Object>> dataOfoneDeviceID = getData(sqlExecuting,deviceID,timeStamp);
//
//                    deviceIDMap.put(deviceID,dataOfoneDeviceID);
//                    dataOfoneTable.add(deviceIDMap);
//                }
//                allTableData.put(tablenName,dataOfoneTable);
//            }
//
//        }
//        System.out.println("旧数据： "+allTableData);
//        System.out.println("===============================");
//
//
//        Map<String,List<Map<String, List<Map<String, Object>>>>> newAllData = new HashMap<>();
//        for(Map.Entry<String, List<Map<String,List<Map<String,Object>>>>> dataEntry : allTableData.entrySet()){
//            List<Map<String, List<Map<String, Object>>>> newRecords = transformFields(dataEntry.getKey(),tableFieldMap,dataEntry.getValue());
//            newAllData.put(dataEntry.getKey(),newRecords);
//        }
//
//        for(String tableName : newAllData.keySet()){
//            List<Map<String, List<Map<String, Object>>>> records = newAllData.get(tableName);
//            String jsonStringData = FormatUtil.list2Json(records);
//            if(MqttUtil.publish2MQTT(jsonStringData)){
//                updateLastRecordTimeSended(tableName,records);
//            }else{
//                System.out.println("消息推送失败");
//            }
//        }
//
//    }

    @Scheduled(initialDelay=1000, fixedRate = 1000 * 3600) //通过@Scheduled声明该方法是计划任务，使用fixedRate属性每隔固定时间执行
    public void reportRecord1(){
        List<String> allTableNames = getAllTableName();
        Map<String,Map<String,String>> tableFieldMap = new HashMap<>();
        Map<String, List<Map<String,List<Map<String,Object>>>>> allTableData = new HashMap<>();
        for(String tableName : allTableNames){
            //if用来测试
            if(!"data_eaif_h".equals(tableName)){
                Map<String,String> fieldMap = getFieldMap(tableName);
                String sqlExecuting = getSQL(tableName);
                Map<String,Object> devIDLastTimeMap = getDeviceID1(tableName);
                for(String deviceID : devIDLastTimeMap.keySet()){
                    List<Map<String,Object>> dataOfoneDeviceID = getData(sqlExecuting,deviceID, (String) devIDLastTimeMap.get(deviceID));

                    List<Map<String,Object>> newDataOfoneDeviceID = transformList(fieldMap,dataOfoneDeviceID);

                    String jsonStringData = FormatUtil.list2Json(newDataOfoneDeviceID);
                    if(MqttUtil.publish2MQTT(jsonStringData)){
                        updateSyncRecordsTable(tableName,deviceID, (String) devIDLastTimeMap.get(deviceID));
                    }else{
                        System.out.println("消息推送失败");
                    }
                }
            }
        }
//        System.out.println("旧数据： "+allTableData);
//        System.out.println("===============================");


//        Map<String,List<Map<String, List<Map<String, Object>>>>> newAllData = new HashMap<>();
//        for(Map.Entry<String, List<Map<String,List<Map<String,Object>>>>> dataEntry : allTableData.entrySet()){
//            List<Map<String, List<Map<String, Object>>>> newRecords = transformFields(dataEntry.getKey(),tableFieldMap,dataEntry.getValue());
//            newAllData.put(dataEntry.getKey(),newRecords);
//        }

//        for(String tableName : newAllData.keySet()){
//            List<Map<String, List<Map<String, Object>>>> records = newAllData.get(tableName);
//            String jsonStringData = FormatUtil.list2Json(records);
//            if(MqttUtil.publish2MQTT(jsonStringData)){
//                updateLastRecordTimeSended(tableName,records);
//            }else{
//                System.out.println("消息推送失败");
//            }
//        }


    }

//    //返回替换字段名的records
//    private List<Map<String,List<Map<String,Object>>>> transformFields(String recordTableName, Map<String,Map<String,String>> tableFieldMap, List<Map<String,List<Map<String,Object>>>> records) {
//        List<Map<String,List<Map<String,Object>>>> newRecords = new ArrayList<>();
//        if(tableFieldMap.containsKey(recordTableName)){
//            for(Map<String,List<Map<String,Object>>> record : records  ){
//                newRecords.add(transformMore(tableFieldMap.get(recordTableName),record));
//            }
//        }
//        return  newRecords;
//    }
//
//    private Map<String,List<Map<String,Object>>> transformMore(Map<String,String> fieldMap, Map<String,List<Map<String,Object>>> deviceIDDataMap) {
//        Map<String,List<Map<String,Object>>> newDeviceIDData = new HashMap<>();
//        for(Map.Entry<String,List<Map<String,Object>>> entry : deviceIDDataMap.entrySet()){
//            newDeviceIDData.put(entry.getKey(),transformMoreAgain(fieldMap, entry.getValue()));
//        }
//        return newDeviceIDData;
//    }
//
//    private List<Map<String,Object>> transformMoreAgain(Map<String,String> fieldMap, List<Map<String,Object>> deviceIDData) {
//        List<Map<String,Object>> newDeviceIDData = new ArrayList<>();
//        for(Map<String,Object> fieldValueMap : deviceIDData){
//            newDeviceIDData.add(transformMoreAgain2(fieldMap,fieldValueMap));
//        }
//        return newDeviceIDData;
//    }
//
//    private Map<String,Object> transformMoreAgain2(Map<String,String> fieldMap, Map<String,Object> fieldValueMap) {
//        Map<String,Object> newFieldValueMap = new HashMap<>();
//        for(String field : fieldMap.keySet()){
//            for(String columnName : fieldValueMap.keySet() ){
//                if(Objects.equals(field,columnName)){
//                    newFieldValueMap.put(fieldMap.get(field),fieldValueMap.get(columnName) );
//                }
//            }
//        }
//        return newFieldValueMap;
//    }

    private List<Map<String,Object>> transformList(Map<String,String> fieldMap, List<Map<String,Object>> deviceIDData) {
        List<Map<String,Object>> newDeviceIDData = new ArrayList<>();
        for(Map<String,Object> fieldValueMap : deviceIDData){
            newDeviceIDData.add(transformOneRecord(fieldMap,fieldValueMap));
        }
        return newDeviceIDData;
    }

    private Map<String,Object> transformOneRecord(Map<String,String> fieldMap, Map<String,Object> fieldValueMap) {
        Map<String,Object> newFieldValueMap = new HashMap<>();
        for(String field : fieldMap.keySet()){
            if(fieldValueMap.containsKey(field)){
                newFieldValueMap.put(fieldMap.get(field),fieldValueMap.get(field) );
            }
        }
        return newFieldValueMap;
    }

//
//    public void updateLastRecordTimeSended(String tableName, List<Map<String,List<Map<String, Object>>>> records) {
//        List<String> deviceIDs = getDeviceID(tableName);
//        for(String deviceID : deviceIDs){
//            String lastRecordTimesJustSended = null;
//            for(Map<String,List<Map<String, Object>>> recordMap : records){
//                if(recordMap.get(deviceID) != null){
//                    List<Map<String, Object>> deviceIDData = recordMap.get(deviceID);
//                    lastRecordTimesJustSended = (String) deviceIDData.get(deviceIDData.size()-1).get("AcquisitionTime");
//                    System.out.println(tableName+"表"+deviceID+"最后一条记录时间: "+lastRecordTimesJustSended);
//                }
//
//            }
////            updateSyncRecordsTable(tableName, deviceID, lastRecordTimesJustSended);
//
//        }
//
//    }


    public boolean updateSyncRecordsTable(String tableName, String deviceID, String time) {
        Connection conn = null;
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        try {
            conn = DataSourceUtils.getConnection();
            String sql = "update sync_records set field_val2 = ? where table_name = ? and devid_val = ?";
            pstmt = conn.prepareStatement(sql);
            pstmt.setString(1, time);
            pstmt.setString(2, tableName);
            pstmt.setString(3, deviceID);
            pstmt.executeUpdate();

        } catch (SQLException e) {
            logger.error("execute sql exception:", e);
            return false;
        } finally {
            DataSourceUtils.closeResource(rs, pstmt, conn);
        }
        return  true;
    }


    @Scheduled(fixedDelay = Long.MAX_VALUE) // 用一个非常大的延迟值，确保只执行一次
    public void subScribeSamle() {
        System.out.println("subScribe执行一次 "+new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));
        MqttUtil.subScribeMQTT();
    }






}

